# File tres/Scripts/trsmed.cl
# December 11, 2008
# By Doug Mink, SAO Telescope Data Center

# TRSMED - Use mscred.combine to compute the median of a list of images

procedure trsmed (images)

string	images=""	{prompt="Images to combine"}

string  outroot=""      {prompt="Root of output file name"}
file	outfile=""	{prompt="Output file name (returned)"}
int	nfiles=0	{prompt="Number of files combined (returned)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

struct *medfiles

begin

file	templist, imout, imin1
int	medseq, jdate
string	tchar, imagelist, history, dstring, oroot, fn1, fn2
file	listfile, imfile
bool	dellist, vb
real	mjdsum, mjdavg, exptot, bcvsum, bcvavg, hasum, stsum
real	exp, mjd, utesum, uteavg, airsum, haavg, stavg, airavg, expavg
real	gmjdsum, gjdavg, hmjdsum, hjdavg, gmjd, hmjd, ut, ute, uts

	vb = verbose
	dellist = deltemp
	imagelist = images
	oroot = outroot

# If input image is a list prepend @ if not already there
	smatch (imagelist, ".list")
	if (smatch.found) {
	    if (substr (imagelist,1,1) != '@')
		imagelist = "@" // images
	    }

	imout = outfile
	fn1 = "0"
	medseq = trssum.sumseq

# Load needed packages if necessary
	if (!defpac ("mscred"))
	    mscred

#  Make temporary list of input files
	templist = mktemp ("tempmed") // ".list"
	files (imagelist, >templist)

# Count files and initialize output root name
	nfiles = 0
	jdate = 0
	medfiles = templist
	exptot = 0.0
	mjdsum = 0.0
	bcvsum = 0.0
	gmjdsum = 0.0
	hmjdsum = 0.0
	airsum = 0.0
	hasum = 0.0
	stsum = 0.0
	ute = 0.0
	uts = 0.0
	while (fscan (medfiles,imin1) != EOF) {
	    nfiles = nfiles + 1
	    if (nfiles == 1) {
		smatch (imin1, ".fits")
		if (oroot == "") {
		    if (smatch.found) {
			oroot = substr (imin1, 6, smatch.start-1)
			}
		    else {
			oroot = substr (imin1, 6, 100)
			}
		    fn1 = substr (imin1,1,4)
		    }
		}
	    tfib1.mjd = 0.0
	    tfib1.exptime = "0.0"
	    tfib1 (imin1)
	    exp = real (tfib1.exptime)
	    exptot = exptot + exp
	    mjd = tfib1.mjd
	    mjdsum = mjdsum + mjd
	    gettime (tfib1.mjd, usesec+)
	    ut = real (gettime.time)
	    if (uts == 0.0 || ut < uts)
		uts = ut
	    imgetp.default = ""
	    imgetp (imin1, "UTEND")
	    if (imgetp.value != "") {
		ut = real (imgetp.value)
		if (ut > ute)
		    ute = ut
		}
	    imgetp.default = ""
	    imgetp (imin1, "GJDN")
	    if (imgetp.value != "") {
		if (jdate == 0)
		    jdate = int (imgetp.value)
		gmjd = real (imgetp.value) - real (jdate)
		gmjdsum = gmjdsum + gmjd
#		print ("GJDN= ",imgetp.value," - ", jdate, " = ", gmjd," -> ",gmjdsum)
		}
	    if (tfib1.imagetype == "OBJECT") {
		imgetp.default = ""
		imgetp (imin1, "ST")
		if (imgetp.value != "")
		    stsum = stsum + real (imgetp.value)
		imgetp.default = ""
		imgetp (imin1, "HA")
		if (imgetp.value != "")
		    hasum = hasum + real (imgetp.value)
		imgetp.default = ""
		imgetp (imin1, "AIR")
		if (imgetp.value != "")
		    airsum = airsum + real (imgetp.value)
		imgetp.default = ""
		imgetp (imin1, "BCV")
		if (imgetp.value != "")
		    bcvsum = bcvsum + real (imgetp.value)
		imgetp.default = ""
		imgetp (imin1, "HJDN")
		if (imgetp.value != "") {
		    if (jdate == 0)
			jdate = int (imgetp.value)
		    hmjd = real (imgetp.value) - real (jdate)
		    hmjdsum = hmjdsum + hmjd
		    }
		}
	    fn2 = substr (imin1,1,4)
	    }

# Return null string as output file if no files were processed
	if (nfiles == 0) {
	    print ("TRSMED: No files to process")
	    outfile = ""
	    return
	    }

# Return input file as output file if only one file
	else if (nfiles == 1) {
	    print ("TRSMED: Only one file, returning it as median file")
	    outfile = imin1
	    return
	    }
        else if (vb) {
            print ("TRSMED is computing median of ",nfiles," files from ", imagelist)
            }

	if (fn1 == "0")
	    imout = oroot // "_med.fits"
	else
	    imout = medseq // "." // oroot // "_med_" // fn1 // "_" // fn2 // ".fits"

	if (vb) {
	    print ("TRSMED: median of ",nfiles," files from ", imagelist," to ", imout)
	    }

	combine.input = "@" // templist
	combine.output = imout
	combine.headers = ""
	combine.bpmasks = ""
	combine.rejmasks = ""
	combine.nrejmasks = ""
	combine.expmasks = ""
	combine.sigmas = ""
	combine.ccdtype = ""
	combine.amps = no
	combine.subsets = no
	combine.delete = no
	combine.combine = "median"
	combine.reject = "ccdclip"
	combine.project = no
	combine.outtype = "real"
	combine.outlimits = ""
	combine.offsets = "none"
	combine.masktype = "none"
	combine.maskvalue = 0.
	combine.blank = 0.
	combine.scale = "exposure"
	combine.zero = "none"
	combine.weight = "none"
	combine.statsec = ""
	combine.lthreshold = INDEF
	combine.hthreshold = INDEF
	combine.nlow = 1
	combine.nhigh = 1
	combine.nkeep = 1
	combine.mclip = yes
	combine.lsigma = 3.
	combine.hsigma = 3.
	combine.rdnoise = "0."
	combine.gain = "1."
	combine.snoise = "0."
	combine.sigscale = 0.1
	combine.pclip = -0.5
	combine.grow = 0.

	combine (imagelist, imout)

# Convert time of exposure to mean of combined files
	mjdavg = mjdsum / real (nfiles)
	round (mjdavg, "%12.6f")
	hedit (imout, "MJD",round.value,update+,verify-,show=vb)
	gettime (mjdavg, usesec+)
	hedit (imout, "DATE-OBS",gettime.fitsdate,update+,verify-,show=vb)

# Start time of exposure is start of the first exposure in sum
	if (uts > 24.0)
	    uts = uts - 24.0
	ut = uts / 24.0
	gettime (ut, usesec+)
	hedit (imout, "UTOPEN", gettime.time, update+, verify-, show=vb)

# End time of exposure is end of the last exposure in sum
	if (ute > 24.0)
	    ute = ute - 24.0
	ut = ute / 24.0
	gettime (ut, usesec+)
	hedit (imout, "UTEND", gettime.time, update+, verify-, show=vb)

	if (tfib1.imagetype == "OBJECT") {

# Convert hour angle to mean of input files
	    haavg = (hasum / real (nfiles)) / 24.0
	    gettime (haavg, usesec+)
	    hedit (imout, "HA",gettime.time,update+,verify-,show=vb)

# Convert sidereal time to mean of input files
	    stavg = (stsum / real (nfiles)) / 24.0
	    gettime (stavg, usesec+)
	    hedit (imout, "ST",gettime.time,update+,verify-,show=vb)

# Convert airmass to mean of input files
	    airavg = airsum / real (nfiles)
	    round (airavg, "%6.3f")
	    hedit (imout, "AIR",round.value,update+,verify-,show=vb)

# Set barycentric velocity correction to mean of those in combined files
	    bcvavg = bcvsum / real (nfiles)
	    round (bcvavg, "%10.5f")
	    hedit (imout, "BCV",round.value,update+,verify-,show=vb)

# Set geocentric Julian Date to mean of those in combined files
	    gjdavg = gmjdsum / real (nfiles)
	    gjdavg = gjdavg + real (jdate)
	    round (gjdavg, "%10.5f")
	    hedit (imout, "GJDN",round.value,update+,verify-,show=vb)

# Set heliocentric Julian Date to mean of those in combined files
	    hjdavg = hmjdsum / real (nfiles)
	    hjdavg = hjdavg + real (jdate)
	    round (hjdavg, "%10.5f")
	    hedit (imout, "HJDN",round.value,update+,verify-,show=vb)
	    }
    
# Update exposure time to mean of those in combined files
	expavg = exptot / real (nfiles)
	round (exptot, "%.1f")
	hedit (imout, "EXPTIME", round.value, update+, verify-,show=vb)

# Make history line including time
	gettime (0.0)
	dstring = gettime.date // " " // gettime.time
	history = dstring // " median of " // nfiles // " images"
	hedit (imout, "TRSMED", history, add+, verify-)

# Increment sequence number for this directory
	trssum.sumseq = trssum.sumseq + 1

	if (dellist) {
	    if (access (templist))
		delete (templist)
	    }
	medfiles = ""
end

# Dec 14 2007	New program for TRES pipeline

# Jan 10 2008	Change name from tresmed to trsmed
# Jan 31 2008	Add TRSMED keyword to output header
# Feb 11 2008	Change temporary file from tempcombine*list to tempmed*.list
# May 16 2008	Add input time argument to gettime() 
# Oct 29 2008	Modify MJD, DATE-OBS, BCV, HJDN, EXPTIME keywords
# Dec  5 2008	Make @ optional if images is *.list file
# Dec 11 2008	Return nfiles parameter with number of files used for median
# Dec 11 2008	Return parameter outfile as null string if no files added
# Dec 11 2008	Return parameter outfile as input file if only one
